@@ -4,7 +4,7 @@ endorse <- function(Y,
                     village = NA,
                     treat = NA,
                     na.strings = 99,
-                    identical.lambda = TRUE,
+                    identical.lambda = FALSE,
                     covariates = FALSE,
                     formula.indiv = NA,
                     hierarchical = FALSE,
@@ -62,11 +62,14 @@ endorse <- function(Y,
 
   mda = FALSE
 
-  if (!identical.lambda & hierarchical)
-    stop("Options are not consistent. If 'identical.lambda = TRUE', 'hierarchical' must be set at
-         'FALSE.'")
+  ## if (!identical.lambda & hierarchical) {
+  ##   stop("Options are not consistent. If 'identical.lambda = TRUE', 'hierarchical' must be set at
+  ##        'FALSE.'")
+  ## }
 
-  if (covariates == FALSE) formula.indiv <- ~ 1
+  if (covariates == FALSE) {
+    formula.indiv <- ~ 1
+  }
 
   cov.mat <- model.matrix(formula.indiv, data)
   M <- ncol(cov.mat)
@@ -78,11 +81,17 @@ endorse <- function(Y,
   #############################################
   data <- data[complete.cases(cov.mat),] ## do NOT work. NOT DROP NA obs
   cov.mat <- cov.mat[complete.cases(cov.mat), ]
-
+  
   N <- nrow(data)
   #############################################
   J <- length(Y)
 
+  if (!is.na(treat[1])) {
+    if (class(treat) != "matrix" | dim(treat)[1] != N | dim(treat)[2] != J) {
+      stop(paste("'treat' must be a ", N, "-by-", J, " matrix.", sep = ""))
+    }
+  }
+
   if (hierarchical) {
     village <- as.integer(as.factor(eval(parse(text = paste("data$", village,
                                                  sep = ""))))) - 1
@@ -194,7 +203,9 @@ endorse <- function(Y,
     }
 
     .Random.seed <- update.start$seed
+    
   } else {
+    
     if (is.na(tau.start[1])) {
       tau.start <- matrix(-99, nrow = J, ncol = max.L)
 
@@ -213,37 +224,43 @@ endorse <- function(Y,
     if (length(x.start) == 1) {
       x.start <- rep(x.start, times = N)
     } else if (length(x.start) != N) {
-      stop(paste("The length of x.start should be", N))
+      stop(paste("The length of 'x.start' should be", N))
     }
 
     if (length(s.start) == 1) {
       s.start <- as.numeric(matrix(s.start, nrow = N, ncol = J))
       s.start[as.integer(endorse) == 0] <- 0
     } else if (class(s.start) != "matrix" | dim(s.start)[1] != N | dim(s.start)[2] != J) {
-      stop(paste("Incorrect input for s.start. It should be a ",
-                 N, "-by-", J, " matrix.", sep = ""))
+      stop(paste("'s.start' should be a ", N, "-by-", J, " matrix.", sep = ""))
     } else {
       s.start <- as.double(s.start)
       s.start[as.integer(endorse) == 0] <- 0
     }
-      
+
     if (length(beta.start) == 1) {
       beta.start <- matrix(beta.start, nrow = J, ncol = 2)
     } else if (class(beta.start) != "matrix" | dim(beta.start)[1] != J | dim(beta.start)[2] != 2) {
-      stop(paste("Incorrect input for beta.start. It should be a ",
+      stop(paste("Incorrect input for 'beta.start'. It should be a ",
                  J, "-by-", 2, " matrix.", sep = ""))
     }
 
     if (length(lambda.start) == 1) {
       lambda.start <- matrix(lambda.start, nrow = G + M, ncol = J * K)
-    } else if (covariates & !identical.lambda & (class(lambda.start) != "matrix" |
-                                                 dim(lambda.start)[1] != M |
-                                                 dim(lambda.start)[2]  != J * K)) {
+    } else if (covariates & !identical.lambda & !hierarchical &
+               (class(lambda.start) != "matrix" | dim(lambda.start)[1] != M |
+                dim(lambda.start)[2]  != J * K)) {
       stop(paste("Incorrect input for lambda.start. It should be a ",
                  M, "-by-", J * K, " matrix.", sep = ""))
-    } else if (!covariates & !identical.lambda & length(lambda.start) != J * K) {
-      stop(paste("Incorrect input for lambda.start. Its length should be ", J * K,
-                 sep = ""))
+    } else if (covariates & !identical.lambda & hierarchical &
+               (class(lambda.start) != "matrix" | dim(lambda.start)[1] != G + M - 1 |
+                dim(lambda.start)[2]  != J * K)) {
+      stop(paste("Incorrect input for lambda.start. It should be a ",
+                 M - 1, "-by-", J * K, " matrix.", sep = ""))
+    } else if (!covariates & !identical.lambda & (class(lambda.start) != "matrix" |
+                                                  dim(lambda.start)[1] != G |
+                                                  dim(lambda.start)[2] != J * K)) {
+      stop(paste("Incorrect input for lambda.start. It should be a ", G, "-by-",
+                 J * K, " matrix.", sep = ""))
     } else if (covariates & identical.lambda & hierarchical &
                (class(lambda.start) != "matrix" | dim(lambda.start)[1] != G + M - 1 |
                 dim(lambda.start)[2] != K)) {
@@ -264,32 +281,81 @@ endorse <- function(Y,
                  sep = ""))
     }
 
+    ## check the dim of starting values for theta
     if (length(theta.start) == 1) {
-      theta.start <- matrix(theta.start, nrow = M, ncol = K)
+      if (hierarchical) {
+        theta.start <- matrix(theta.start, nrow = M - 1 + P, ncol = K)
+      } else {
+        theta.start <- matrix(theta.start, nrow = M, ncol = K)
+      }
+    } else if (hierarchical & !identical.lambda &
+               (class(theta.start) != "matrix" | dim(theta.start)[1] != M - 1 + P |
+                dim(theta.start)[2] != K)) {
+      stop(paste("'theta.start' should be a ", M - 1 + P, "-by-", K, " matrix.", sep = ""))
+    } else if (!hierarchical & !identical.lambda &
+               (class(theta.start) != "matrix" | dim(theta.start)[1] != M |
+                dim(theta.start)[2] != K)) {
+      stop(paste("'theta.start' should be a ", M, "-by-", K, " matrix.", sep = ""))
     }
 
+
+    ## check the dim of starting values for phi2
     if (length(phi2.start) == 1) {
-      phi2.start <- rep(phi2.start, times = K * M)
+      if (hierarchical) {
+        phi2.start <- rep(phi2.start, times = K * (M - 1 + P))
+      } else {
+        phi2.start <- rep(phi2.start, times = K * M)
+      }
+    } else if (hierarchical & !identical.lambda &
+               (class(phi2.start) != "matrix" | dim(phi2.start)[1] != M - 1 + P |
+                dim(phi2.start)[2] != K)) {
+      stop(paste("'phi2.start' should be a ", M - 1 + P, "-by-", K, " matrix.", sep = ""))
+    } else if (!hierarchical & !identical.lambda &
+               (class(phi2.start) != "matrix" | dim(phi2.start)[1] != M |
+                dim(phi2.start)[2] != K)) {
+      stop(paste("'phi2.start' should be a ", M, "-by-", K, " matrix.", sep = ""))
     }
 
-  if (hierarchical) {
-    if (length(kappa.start) == 1) {
-      kappa.start <- rep(kappa.start, times = K * P)
-    } else if (class(kappa.start) != "matrix" | dim(kappa.start)[1] != P |
-               dim(kappa.start)[2] != K) {
-      stop(paste("Incorrect input for kappa.start. It should be a ", P, "-by", K, " matrix.",
-                 sep = ""))
-    }
 
-    if (length(psi2.start == 1)) {
-      psi2.start <- rep(psi2.start, times = K)
-    }
+    ## check the dim of starting values for kappa, psi2, zeta
+    if (hierarchical) {
+      ## kappa
+      if (length(kappa.start) == 1) {
+        if (identical.lambda) {
+          kappa.start <- rep(kappa.start, times = K * P)
+        } else {
+          kappa.start <- rep(kappa.start, times = J * K * P)
+        }
+      } else if (identical.lambda & (class(kappa.start) != "matrix" | dim(kappa.start)[1] != P |
+                                     dim(kappa.start)[2] != K)) {
+        stop(paste("Incorrect input for kappa.start. It should be a ", P, "-by", K, " matrix.",
+                   sep = ""))
+      } else if (!identical.lambda & (class(kappa.start) != "matrix" | dim(kappa.start)[1] != P |
+                                      dim(kappa.start)[2] != J * K)) {
+        stop(paste("Incorrect input for kappa.start. It should be a ", P, "-by", J * K, " matrix.",
+                   sep = ""))
+      }
 
-    if (length(zeta.start) == 1) {
-      zeta.start <- rep(zeta.start, times = P)
-    }
+      ## psi2
+      if (length(psi2.start == 1)) {
+        if (identical.lambda) {
+          psi2.start <- rep(psi2.start, times = K)
+        } else {
+          psi2.start <- rep(psi2.start, times = J*K)
+        }
+      } else if (identical.lambda & length(psi2.start) != K){
+        stop(paste("'psi2.start' should be a vector of length ", K, ".", sep = ""))
+      } else if (!identical.lambda & length(psi2.start) != J * K){
+        stop(paste("'psi2.start' should be a vector of length ", J * K, ".", sep = ""))
+      }
 
-  }
+      ## zeta
+      if (length(zeta.start) == 1) {
+        zeta.start <- rep(zeta.start, times = P)
+      } else if (length(zeta.start) != P) {
+        stop(paste("'zeta.start' should be a vector of length ", P, ".", sep = ""))
+      }
+    }
 
     if (length(delta.start) == 1) {
       if (hierarchical) {
@@ -299,28 +365,51 @@ endorse <- function(Y,
       }
     } else {
       if (hierarchical & length(delta.start) != (G + M - 1)) {
-      	 stop(paste("Incorrect input for delta.start. Its length should be ", G + M - 1,
-                 sep = ""))
+        stop(paste("Incorrect input for delta.start. Its length should be ", G + M - 1,
+                   sep = ""))
       } else if (!hierarchical & length(delta.start) != M) {
-      	 stop(paste("Incorrect input for delta.start. Its length should be ", M,
-                 sep = ""))
+        stop(paste("Incorrect input for delta.start. Its length should be ", M,
+                   sep = ""))
       }
     }
   }
-  
+
+###
+### check the dimension of prior parameters
+###
+
+  ## prior mean of beta
   if (length(mu.beta) == 1) {
     mu.beta <- matrix(mu.beta, nrow = J, ncol = 2)
+  } else if (class(mu.beta) != "matrix" | dim(mu.beta)[1] != J |
+             dim(mu.beta)[2] != 2) {
+    stop(paste("'mu.beta' should be a ", J, "-by-", 2, " matrix.", sep = ""))
   }
 
+  ## prior mean of theta  (ADD 'mu.lambda' in the next update)
   if (length(mu.theta) == 1) {
-    mu.theta <- rep(mu.theta, times = M * K)
+    mu.theta <- rep(mu.theta, times = (M - 1 + P) * K)
   } else if (identical.lambda & hierarchical & (class(mu.theta) != "matrix" |
                                                 dim(mu.theta)[1] != (M - 1) |
                                                 dim(mu.theta)[2] != K)) {
-    stop(paste("Incorrect input for mu.theta. It should be a ", M - 1, "-by-", K, " matrix.",
+    stop(paste("'mu.theta' (prior mean of lambda) has incorrect dimensions. It should be a ",
+               M - 1, "-by-", K, " matrix.",
+               sep = ""))
+  } else if (!identical.lambda & hierarchical & (class(mu.theta) != "matrix" |
+                                                 dim(mu.theta)[1] != M - 1 + P |
+                                                 dim(mu.theta)[2] != K)) {
+    stop(paste("'mu.theta' (prior mean of theta) has incorrect dimensions. It should be a ",
+               M - 1 + P, "-by-", K, " matrix.",
+               sep = ""))
+  } else if (!identical.lambda & !hierarchical & (class(mu.theta) != "matrix" |
+                                                  dim(mu.theta)[1] != M |
+                                                  dim(mu.theta)[2] != K)) {
+    stop(paste("'mu.theta' (prior mean of theta) has incorrect dimensions. It should be a ",
+               M, "-by-", K, " matrix.",
                sep = ""))
   }
 
+  ## prior mean of delta
   if (covariates) {
     if (length(mu.delta) == 1) {
       if (hierarchical) {
@@ -328,23 +417,31 @@ endorse <- function(Y,
       } else {
         mu.delta <- rep(mu.delta, times = M)
       }
+    } else if (length(mu.delta) != M - 1 + G){
+      stop(paste("'mu.delta' must be a vector of length ", M - 1 + G, ".", sep = ""))
     }
   } else {
     if (length(mu.x) == 1) {
       mu.delta <- rep(mu.x, times = N)
-    } else {
+    } else if (length(mu.x) == N) {
       mu.delta <- mu.x
+    } else if (length(mu.x) != N) {
+      stop(paste("'mu.x' must be a vector of length ", N, ".", sep = ""))
     }
   }
+      
 
   if (hierarchical) {
+    ## prior mean of kappa
     if (length(mu.kappa) == 1) {
       mu.kappa <- rep(mu.kappa, times = K * P)
-    } else if (class(mu.kappa) != "matrix" | dim(mu.kappa)[1] != P | dim(mu.kappa)[2] != K){
+    } else if (identical.lambda & (class(mu.kappa) != "matrix" | dim(mu.kappa)[1] != P |
+                                   dim(mu.kappa)[2] != K)){
       stop(paste("Incorrect input for mu.kappa. It should be a ", P, "-by-", K, " matrix.",
                  sep = ""))
     }
 
+    ## prior mean of zeta
     if (length(mu.zeta) == 1) {
       mu.zeta <- rep(mu.zeta, times = P)
     } else if (length(mu.zeta) != P){
@@ -353,21 +450,65 @@ endorse <- function(Y,
     }
   }
 
-  precision.beta <- diag(precision.beta, nrow = 2, ncol = 2)
-
-  if (hierarchical)
-    precision.delta <- diag(precision.delta, nrow = M - 1, ncol = M - 1)
-  else
-    precision.delta <- diag(precision.delta, nrow = M, ncol = M)
-  
-
-  if (length(precision.theta) == 1) {
-    precision.theta <- rep(precision.theta, times = M)
+  ## prior precision of beta
+  if (length(precision.beta) == 1) {
+    precision.beta <- diag(precision.beta, nrow = 2, ncol = 2)
+  } else if (class(precision.beta) != "matrix" | dim(precision.beta)[1] != 2 |
+             dim(precision.beta)[2] != 2) {
+    stop("'precision.beta' must be a 2-by-2 matrix.")
   }
 
+  ## prior precision of delta
   if (hierarchical) {
-    precision.kappa <- diag(precision.kappa, nrow = P, ncol = P)
-    precision.zeta <- diag(precision.zeta, nrow = P, ncol = P)
+    if (length(precision.delta) == 1) {
+      precision.delta <- diag(precision.delta, nrow = M - 1, ncol = M - 1)
+    } else if (class(precision.delta) != "matrix" | dim(precision.delta)[1] != M - 1 |
+               dim(precision.delta)[2] != M - 1) {
+      stop(paste("'precision.delta' must be a ", M - 1, "-by", M - 1, " matrix.", sep = ""))
+    }
+  } else {
+    if (length(precision.delta) == 1) {
+      precision.delta <- diag(precision.delta, nrow = M, ncol = M)
+    } else if (class(precision.delta) != "matrix" | dim(precision.delta)[1] != M |
+               dim(precision.delta)[2] != M) {
+      stop(paste("'precision.delta' must be a ", M, "-by", M, " matrix.", sep = ""))
+    }
+  }
+
+  ## prior precision of theta (lambda)
+  if (length(precision.theta) == 1) {
+    precision.theta <- rep(precision.theta, times = M + P)
+  } else if (hierarchical & identical.lambda & length(precision.theta) != M - 1) {
+    stop(paste("'precision.theta' (prior precision of lambda) must be a vector of length ",
+               M - 1, ".", sep = ""))
+  } else if (hierarchical & !identical.lambda & length(precision.theta) != M - 1 + P) {
+    stop(paste("'precision.theta' must be a vector of length ", M - 1 + P, ".", sep = ""))
+  } else if (!hierarchical & identical.lambda & length(precision.theta) != M) {
+    stop(paste("'precision.theta' (prior precision of lambda) must be a vector of length ",
+               M, ".", sep = ""))
+  } else if (!hierarchical & !identical.lambda & length(precision.theta) != M) {
+    stop(paste("'precision.theta' must be a vector of length ", M, ".", sep = ""))
+  }
+
+
+  if (hierarchical) {
+    ## prior precision of kappa
+    if (length(precision.kappa) == 1) {
+      precision.kappa <- diag(precision.kappa, nrow = P, ncol = P)
+    } else if (hierarchical & identical.lambda & (class(precision.kappa) != "matrix" |
+                                                  dim(precision.kappa)[1] != P |
+                                                  dim(precision.kappa)[2] != P)) {
+      stop(paste("'precision.kappa' must be a ", P, "-by-", P, " matrix.", sep = ""))
+    }
+
+    ## prior precision of zeta
+    if (length(precision.zeta) == 1) {
+      precision.zeta <- diag(precision.zeta, nrow = P, ncol = P)
+    } else if (hierarchical & (class(precision.zeta) != "matrix" | dim(precision.zeta)[1] != P |
+                               dim(precision.zeta)[2] != P)) {
+      stop(paste("'precision.zeta' must be a ", P, "-by-", P, " matrix.", sep = ""))
+    }
+    
   }
 
   if (length(prop) != J) {
@@ -410,7 +551,7 @@ endorse <- function(Y,
              as.double(mu.zeta),
              as.double(precision.zeta),
              as.double(c(s0.omega2, nu0.omega2, s0.phi2, nu0.phi2, s0.psi2, nu0.psi2, s0.sig2,
-                       nu0.sig2, s0.rho2, nu0.rho2)),
+                         nu0.sig2, s0.rho2, nu0.rho2)),
              as.integer(c(mda, mh, x.sd, tau.out, s.out, omega2.out, phi2.out,
                           psi2.out, verbose, seed.store, covariates,
                           identical.lambda, hierarchical, MCMC, burn, thin)),
@@ -419,28 +560,28 @@ endorse <- function(Y,
              tauStore = if (tau.out) double(printout * (max.L - 1) * J) else double(1),
              xStore = if (x.sd) double(printout) else double(printout * N),
              sStore = if (s.out) double(printout * N * J) else double(1),
-             lambdaStore = if (!identical.lambda) double(printout * J * K * M) else if (hierarchical) double(printout * K * (G + M - 1)) else double(printout * K * M),
-             thetaStore = if (!identical.lambda) double(printout * K * M) else double(1),
-             kappaStore = if (hierarchical) double(printout * K * P) else double(1),
+             lambdaStore = if (!hierarchical & !identical.lambda) double(printout * J * K * M) else if (hierarchical & identical.lambda) double(printout * K * (G + M - 1)) else if (!hierarchical & identical.lambda) double(printout * K * M) else if (hierarchical & !identical.lambda) double(printout * J * K * (G + M - 1)),
+             thetaStore = if (hierarchical & !identical.lambda) double(printout * K * (M - 1 + P)) else if (!hierarchical & !identical.lambda) double(printout * K * M) else double(1),
+             kappaStore = if (hierarchical & !identical.lambda) double(printout * J * K * P) else if (hierarchical & identical.lambda) double(printout * K * P) else double(1),
              deltaStore = if (hierarchical) double(printout * (G + M - 1)) else if (covariates) double(printout * M) else double(1),
              zetaStore = if (hierarchical) double(printout * P) else double(1),
              omega2Store = if (omega2.out & !identical.lambda) double(printout * J * K) else if (omega2.out) double(printout * K) else double(1),
-             phi2Store = if (phi2.out & !identical.lambda) double(printout * K * M) else double(1),
-             psi2Store = if (psi2.out & hierarchical) double(printout * K) else double(1),
+             phi2Store = if (phi2.out & !identical.lambda & hierarchical) double(printout * K * (M - 1 + P)) else if (phi2.out & !identical.lambda & !hierarchical) double(printout * K * M) else double(1),
+             psi2Store = if (psi2.out & hierarchical & identical.lambda) double(printout * K) else if (psi2.out & hierarchical & !identical.lambda) double(printout * J * K) else double(1),
 	     sig2Store = if (covariates | hierarchical) double(printout) else double(1),
              rho2Store = if (hierarchical) double(printout) else double(1), 
              betaLast = if (seed.store) double(2 * J) else double(1),
              tauLast = if (seed.store) double((max.L - 1) * J) else double(1),
              xLast = if (seed.store) double(N) else double(1),
              sLast = if (seed.store) double(N * J) else double(1),
-             lambdaLast = if (seed.store & !identical.lambda) double(J * K * M) else if (seed.store) double(K * (M + G - 1)) else double(1),
-             thetaLast = if (seed.store & !identical.lambda) double(K * M) else double(1),
-             kappaLast = if (seed.store & hierarchical) double(K * P) else double(1),
+             lambdaLast = if (seed.store & !identical.lambda & !hierarchical) double(J * K * M) else if (seed.store & !identical.lambda & hierarchical) double(K * J * (M + G - 1)) else if (seed.store & identical.lambda & !hierarchical) double(K * M) else if (seed.store & identical.lambda & hierarchical) double(K * (M + G - 1)) else double(1),
+             thetaLast = if (seed.store & !identical.lambda & !hierarchical) double(K * M) else if (seed.store & !identical.lambda & hierarchical) double(K * (M - 1 + P)) else double(1),
+             kappaLast = if (seed.store & identical.lambda & hierarchical) double(K * P) else if (seed.store & !identical.lambda & hierarchical) double(J * K * P) else double(1),
              deltaLast = if (seed.store & hierarchical) double(G + M - 1) else if (seed.store & !hierarchical) double (M) else double(1),
              zetaLast = if (seed.store & hierarchical) double(P) else double(1),
              omega2Last = if (seed.store & !identical.lambda) double(J * K) else if (seed.store) double(K) else double(1),
              phi2Last = if (seed.store) double(K * M) else double(1),
-             psi2Last = double(K),
+             psi2Last = if (seed.store & identical.lambda & hierarchical) double(K) else if (seed.store & !identical.lambda & hierarchical) double(J * K) else double(1),
 	     sig2Last = double(1),
              rho2Last = double(1),
              accept.ratio = double(J),
@@ -457,14 +598,14 @@ endorse <- function(Y,
                   ncol = N, nrow = printout),
               s = if (s.out) matrix(as.double(temp$sStore), byrow = TRUE,
                 ncol = N * J, nrow = printout) else NULL,
-              lambda = if (identical.lambda & hierarchical) matrix(as.double(temp$lambdaStore), byrow = TRUE, ncol = (G + M - 1) * K, nrow = printout) else if (identical.lambda) matrix(as.double(temp$lambdaStore), byrow = TRUE, ncol = M * K, nrow = printout) else matrix(as.double(temp$lambdaStore), byrow = TRUE, ncol = J * M * K, nrow = printout),
-              theta = if (identical.lambda) NULL else matrix(as.double(temp$thetaStore), byrow = TRUE, ncol = K * M, nrow = printout),
-              kappa = if (hierarchical) matrix(as.double(temp$kappaStore), byrow = TRUE, ncol = P * K, nrow = printout) else NULL,
+              lambda = if (identical.lambda & hierarchical) matrix(as.double(temp$lambdaStore), byrow = TRUE, ncol = (G + M - 1) * K, nrow = printout) else if (identical.lambda & !hierarchical) matrix(as.double(temp$lambdaStore), byrow = TRUE, ncol = M * K, nrow = printout) else if (!identical.lambda & hierarchical) matrix(as.double(temp$lambdaStore), byrow = TRUE, ncol = (G + M - 1) * J * K, nrow = printout) else if (!identical.lambda & !hierarchical) matrix(as.double(temp$lambdaStore), byrow = TRUE, ncol = J * M * K, nrow = printout),
+              theta = if (identical.lambda) NULL else if (hierarchical) matrix(as.double(temp$thetaStore), byrow = TRUE, ncol = K * (M - 1 + P), nrow = printout) else matrix(as.double(temp$thetaStore), byrow = TRUE, ncol = K * M, nrow = printout),
+              kappa = if (hierarchical & identical.lambda) matrix(as.double(temp$kappaStore), byrow = TRUE, ncol = P * K, nrow = printout) else if (hierarchical) matrix(as.double(temp$kappaStore), byrow = TRUE, ncol = P * K * J, nrow = printout) else NULL,
               delta = if (hierarchical) matrix(as.double(temp$deltaStore), byrow = TRUE, ncol = G + M - 1, nrow = printout) else if (covariates) matrix(as.double(temp$deltaStore), byrow = TRUE, ncol = M, nrow = printout) else NULL,
               zeta = if (hierarchical) matrix(as.double(temp$zetaStore), byrow = TRUE, ncol = P, nrow = printout) else NULL,
               omega2 = if (identical.lambda & omega2.out) matrix(as.double(temp$omega2Store), byrow = TRUE, ncol = K, nrow = printout) else if (omega2.out) matrix(as.double(temp$omega2Store), byrow = TRUE, ncol = J * K, nrow = printout) else NULL,
-              phi2 = if (!identical.lambda & phi2.out) matrix(as.double(temp$phi2Store), byrow = TRUE, ncol = K * M, nrow = printout) else NULL,
-              psi2 = if (hierarchical & psi2.out) matrix(as.double(temp$psi2Store), byrow = TRUE, ncol = K, nrow = printout) else NULL,
+              phi2 = if (!identical.lambda & hierarchical & phi2.out) matrix(as.double(temp$phi2Store), byrow = TRUE, ncol = K * (M - 1 + P), nrow = printout) else if (!identical.lambda & !hierarchical & phi2.out) matrix(as.double(temp$phi2Store), byrow = TRUE, ncol = K * M, nrow = printout) else NULL,
+              psi2 = if (hierarchical & identical.lambda & psi2.out) matrix(as.double(temp$psi2Store), byrow = TRUE, ncol = K, nrow = printout) else if (hierarchical & !identical.lambda & psi2.out) matrix(as.double(temp$psi2Store), byrow = TRUE, ncol = K * J, nrow = printout) else NULL,
 	      sig2 = if (hierarchical | covariates) matrix(as.double(temp$sig2Store), ncol = 1, nrow = printout) else NULL,
               rho2 = if (hierarchical) matrix(as.double(temp$rho2Store), byrow = TRUE, ncol = 1, nrow = printout) else NULL,
               accept.ratio = if (mh) as.double(temp$accept.ratio) else NULL,
@@ -473,8 +614,8 @@ endorse <- function(Y,
               tau.start = if (seed.store) as.double(temp$tauLast) else NULL,
               x.start = if (seed.store) as.double(temp$xLast) else NULL,
               s.start = if (seed.store) matrix(as.double(temp$sLast), nrow = N, ncol = J, byrow = TRUE) else NULL,
-              lambda.start = if (seed.store & covariates) as.double(temp$lambdaLast) else NULL,
-              theta.start = if (seed.store & !identical.lambda & covariates) as.double(temp$thetaLast) else NULL,
+              lambda.start = if (seed.store) as.double(temp$lambdaLast) else NULL,
+              theta.start = if (seed.store & !identical.lambda) as.double(temp$thetaLast) else NULL,
               kappa.start = if (seed.store & hierarchical) as.double(temp$kappaLast) else NULL,
               delta.start = if (seed.store & (covariates | hierarchical)) as.double(temp$deltaLast) else NULL,
               zeta.start = if (seed.store & hierarchical) as.double(temp$zetaLast) else NULL,
@@ -522,58 +663,97 @@ endorse <- function(Y,
   if (identical.lambda) {
     if (hierarchical) {
       if (covariates) {
-        colnames(res$lambda) <- paste(rep(c(paste("village", 1:G, sep = "."), var.names.indiv[2:M]), times = K),
+        colnames(res$lambda) <- paste(rep(c(paste("group", 1:G, sep = ""), var.names.indiv[2:M]), times = K),
                                       rep(1:K, each = G + M - 1), sep = ".")
       } else {
-        colnames(res$lambda) <- paste(rep(paste("village", 1:G, sep = "."), times = K),
+        colnames(res$lambda) <- paste(rep(paste("group", 1:G, sep = ""), times = K),
                                       rep(1:K, each = G), sep = ".")
       }
     } else {
       colnames(res$lambda) <- paste(rep(var.names.indiv, times = K),
                                     rep(1:K, each = M), sep =".")
     }
+  } else {
+    if (hierarchical) {
+      temp.village.label <- paste("group", 1:G, sep = "")
+      if (covariates) {
+        temp.col.names <- paste(c(temp.village.label, var.names.indiv[2:M]),
+                                rep(1:J, each = (M + G - 1) * K), sep = ".")
+        colnames(res$lambda) <- paste(temp.col.names, rep(rep(1:K, each = G + M - 1), times = J),
+                                      sep = ".")
+      } else {
+        temp.col.names <- paste(temp.village.label, rep(1:J, each = G * K), sep = ".")
+        colnames(res$lambda) <- paste(temp.col.names, rep(rep(1:K, each = G), times = J),
+                                      sep = ".")
+      }
+    } else {
+      temp.col.names <- paste(var.names.indiv, rep(1:J, each = M * K), sep = ".")
+      colnames(res$lambda) <- paste(temp.col.names, rep(rep(1:K, each = M), times = J), sep = ".")
+    }
   }
+
   res$lambda <- mcmc(res$lambda, start = burn + 1, end = MCMC, thin = thin)
 
 
   if (identical.lambda) {
-    colnames(res$omega2) <- paste("omega2.", 1:K, sep = "")
+    colnames(res$omega2) <- paste("omega2", 1:K, sep = ".")
   } else {
-    colnames(res$omega2) <- paste("omega2.", rep(1:J, each = K), rep(1:K,
-                                                        times = J), sep = "")
+    colnames(res$omega2) <- paste("omega2", rep(1:J, each = K), rep(1:K,
+                                                       times = J), sep = ".")
   }
   res$omega2 <- mcmc(res$omega2, start = burn + 1, end = MCMC, thin = thin)  
 
-  if (identical.lambda == FALSE) { 
-    temp.names <- paste("theta", 1:K, sep = "")
-    if (covariates) {
-      colnames(res$theta) <- paste(rep(temp.names, each = M), rep(1:M, times = K),
-                                   sep = ".")
+
+  if (!identical.lambda) {
+    if (hierarchical) {
+      temp.col.names <- c(rep("indiv", times = (M - 1)), rep("group", times = P))
+      if (M > 1) {
+        temp.col.names <- paste(temp.col.names, c(var.names.indiv[2:M], var.names.village),
+                                sep = ".")
+        temp.col.names <- paste(temp.col.names, rep(1:K, each = (M - 1) + P), sep = ".")
+      } else {
+        temp.col.names <- paste(temp.col.names, rep(var.names.village, times = K),
+                                rep(1:K, each = P), sep = ".")
+      }
+      colnames(res$theta) <- temp.col.names
     } else {
-      colnames(res$theta) <- temp.names
+      if (covariates) {
+        temp.col.names <- paste(var.names.indiv, rep(1:K, each = M), sep = ".")
+        colnames(res$theta) <- temp.col.names
+      } else {
+        temp.col.names <- paste("theta", 1:K, sep = "")
+        colnames(res$theta) <- temp.col.names
+      }
+
+      temp.col.names <- paste("phi2", temp.col.names, sep = ".")
+      colnames(res$phi2) <- temp.col.names
     }
     res$theta <- mcmc(res$theta, start = burn + 1, end = MCMC, thin = thin)
-
-    temp.names <- paste("phi2.", 1:K, sep = "")
-    if (covariates) {
-      colnames(res$phi2) <- paste(rep(temp.names, each = M), rep(1:M, times = K),
-                                  sep = ".")
-    } else {
-      colnames(res$phi2) <- temp.names
-    }
     res$phi2 <- mcmc(res$phi2, start = burn + 1, end = MCMC, thin = thin)
   }
 
   if (hierarchical) {
-    colnames(res$kappa) <- paste(rep(var.names.village, times = K),
-                                 rep(1:K, each = P), sep = ".")
-    res$kappa <- mcmc(res$kappa, start = burn + 1, end = MCMC, thin = thin)
-    if (psi2.out) {
-      colnames(res$psi2) <- paste("psi2", 1:K, sep = ".")
-      res$psi2 <- mcmc(res$psi2, start = burn + 1, end = MCMC, thin = thin)
+    if (identical.lambda) {
+      colnames(res$kappa) <- paste(rep(var.names.village, times = K),
+                                   rep(1:K, each = P), sep = ".")
+      res$kappa <- mcmc(res$kappa, start = burn + 1, end = MCMC, thin = thin)
+      if (psi2.out) {
+        colnames(res$psi2) <- paste("psi2", 1:K, sep = ".")
+        res$psi2 <- mcmc(res$psi2, start = burn + 1, end = MCMC, thin = thin)
+      }
+    } else {
+      colnames(res$kappa) <- paste(rep(var.names.village, times = J * K),
+                                   rep(rep(1:J, each = P), times = K),
+                                   rep(1:K, each = J * P), sep = ".")
+      res$kappa <- mcmc(res$kappa, start = burn + 1, end = MCMC, thin = thin)
+      if (psi2.out) {
+        colnames(res$psi2) <- paste("psi2",
+                                    rep(1:J, times = K),
+                                    rep(1:K, each = J), sep = ".")
+        res$psi2 <- mcmc(res$psi2, start = burn + 1, end = MCMC, thin = thin)
+      }
     }
   }
-  
 
   if (hierarchical) {
     if (covariates) {
@@ -583,8 +763,8 @@ endorse <- function(Y,
     }
     res$delta <- mcmc(res$delta, start = burn + 1, end = MCMC, thin = thin)  
   } else if (covariates) {
-      colnames(res$delta) <- var.names.indiv
-      res$delta <- mcmc(res$delta, start = burn + 1, end = MCMC, thin = thin)  
+    colnames(res$delta) <- var.names.indiv
+    res$delta <- mcmc(res$delta, start = burn + 1, end = MCMC, thin = thin)  
   }
 
   if (hierarchical) {
@@ -602,4 +782,4 @@ endorse <- function(Y,
   names(res$accept.ratio) <- paste("Q", 1:J, sep = "")
 
   return(res)
-}
+}  
